from qt import *
import sys

if __name__ == '__main__':
	import gettext
	gettext.install('bittorrent', 'locale')
	sys.path.append('..')

from config import qtorrent_options, Config, MYTRUE, MYFALSE, MyBool

HIDE_OPTIONS = \
    [
	'display_interval',
    ]

ADDITIONS = \
    {
	'min_uploads': ('max_uploads', 6, _("the maximum number of uploads to allow at once.")),
    }

class SettingsPanel(QWidget):
	def __init__(self, parent, options, config = {}):
		QWidget.__init__(self, parent)
		self.setCaption('QTorrent settings')
		
		options = options[:]
		i = 0
		while i < len(options):
			if options[i][0] in ADDITIONS:
				options.insert(i + 1, ADDITIONS[options[i][0]])
			i += 1
		
		self.grid = grid = QGridLayout(self, len(options) + 1, 3)
		grid.setSpacing(5)
		self.tooltips = QToolTip(self)
		
		self.widgets = {}
		self.revert_lambda = []
		
		i = 0
		for (key, default, descr) in options:
			if key in HIDE_OPTIONS:
				continue
			value = config.get(key, default)
			label = key.replace('_', ' ').capitalize()
			if default in (MYTRUE, MYFALSE):
				w = QCheckBox(label, self)
				grid.addMultiCellWidget(w, i, i, 0, 1)
				if value:
					w.setChecked(True)
				else:
					w.setChecked(False)
				r = lambda w=w, d=default: w.setChecked(d == MYTRUE)
			else:
				l = QLabel(label + ":", self)
				self.tooltips.add(l, descr.capitalize())
				grid.addWidget(l, i, 0)
				if type(default) in (int, float):
					w = QSpinBox(0, sys.maxint, 1, self)
					w.setValue(value)
					r = lambda w=w, d=default: w.setValue(d)
				elif type(default) == str:
					w = QLineEdit(value, self)
					r = lambda w=w,d=default: w.setText(d)
				else:
					raise TypeError('Invalid option type %s' % type(default).__name__)
					w = None
				grid.addWidget(w, i, 1)
			
			self.widgets[key] = (w, type(default))
			self.tooltips.add(w, descr.capitalize())
			
			self.revert_lambda.append(r)
			w = QToolButton(self)
			w.setUsesTextLabel(True)
			w.setTextLabel('X')
			self.tooltips.add(w, 'Revert to default')
			self.connect(w, SIGNAL("clicked()"), r)
			grid.addWidget(w, i, 2)
			
			i += 1
		
		spacer = QSpacerItem(1, 1, QSizePolicy.Minimum, QSizePolicy.Expanding)
		grid.addMultiCell(spacer, i, i, 0, 1)
	
	def fetch(self, config):
		for key, (widget, opttype) in self.widgets.items():
			if opttype == MyBool:
				if widget.isChecked():
					config[key] = MYTRUE
				else:
					config[key] = MYFALSE
			elif opttype == int:
				config[key] = int(widget.value())
			elif opttype == float:
				config[key] = float(widget.value())
			else:
				config[key] = str(widget.text())

class SettingsWindow(QMainWindow):
	def __init__(self, config):
		QMainWindow.__init__(self)
		
		from BitTorrent import defaultargs
		
		vbox = QVBox(self)
		tabs = QTabWidget(vbox)
		
		self.other = SettingsPanel(tabs, qtorrent_options, config)
		tabs.addTab(self.other, "QTorrent")
		self.common = SettingsPanel(tabs, defaultargs.common_options, config)
		tabs.addTab(self.common, "Network")
		self.advanced = SettingsPanel(tabs, defaultargs.rare_options, config)
		tabs.addTab(self.advanced, "Advanced")
		
		group = QHButtonGroup(vbox)
		group.setFrameShape(QFrame.NoFrame)
		b = QPushButton('Ok', group)
		self.connect(b, SIGNAL('clicked()'), self.ok)
		b = QPushButton('Cancel', group)
		self.connect(b, SIGNAL('clicked()'), self.cancel)
		
		self.setCentralWidget(vbox)
	
	def fetch(self, config):
		for tab in self.common, self.other, self.advanced:
			tab.fetch(config)
	
	def ok(self):
		self.hide()
		self.emit(PYSIGNAL('ok'), ())
	
	def cancel(self):
		self.hide()
		self.emit(PYSIGNAL('cancel'), ())
	
	def closeEvent(self, e):
		self.cancel()
	
if __name__ == '__main__':
	def saveAndExit():
		win.fetch(config)
#		config.save()
		app.quit()
	config = Config()
	app = QApplication(sys.argv)
	win = SettingsWindow(config)
	app.setMainWidget(win)
	app.connect(win, PYSIGNAL('ok'), saveAndExit)
	app.connect(win, PYSIGNAL('cancel'), app.quit)
	win.show()
	app.exec_loop()
